
/*
	Swipe2Refresh

	Functionality description
	=============
	1. detect touch on evement
	2. if vertical direction more than horizontal - then calculate deriction

	Usage example
	=============

	Container:
	@onDomRefresh = () ->
		 * Enable swipe 2 refresh
		@s2r = new Swipe2Refresh document.body,
			onSwipeTop: () =>
				app.screens.current.reload()
 */
define(['modernizr'], function(Modernizr) {
  var Swipe2Refresh;
  return Swipe2Refresh = (function() {
    function Swipe2Refresh(el, onSwipeTop) {
      this.el = el;
      this.$el = $(this.el);
      this.onSwipeTop = onSwipeTop;
      this.init();
    }


    /**
    		 * Function checks if touch events are supported by modernizr detection and initialize proper events
    		 * @param {obj} data - orbitary data can be passed here to extend method functionality
     */

    Swipe2Refresh.prototype.init = function(data) {
      this.active = false;
      this.triggered = false;
      this.swipeMinPercent = 2;
      this.swipeReloadPercent = 25;
      this.swipeMaxPercent = 0;
      this.swipeStartPositionPercent = 20;
      this.screenHeight = window.innerHeight;
      this.screenWidth = window.innerWidth;
      this.transitionOffset = Math.round(this.swipeMinPercent * this.screenHeight / 100);
      this.loader = document.getElementById('swipe-loader');
      if (window.navigator.pointerEnabled != null) {
        this.start = 'pointerdown';
        this.move = 'pointermove';
        this.end = 'pointerup';
        this.cancel = 'pointerout';
      } else if (window.navigator.msPointerEnabled != null) {
        this.start = 'MSPointerDown';
        this.move = 'MSPointerMove';
        this.end = 'MSPointerUp';
        this.cancel = 'MSPointerOut';
      } else if (Modernizr.touch) {
        this.start = 'touchstart';
        this.move = 'touchmove';
        this.end = 'touchend';
        this.cancel = 'touchcancel';
      }
      this.touchStart = this.touchStart.bind(this);
      this.touchMove = this.touchMove.bind(this);
      this.touchEnd = this.touchEnd.bind(this);
      this.el.addEventListener(this.start, this.touchStart, false);
      this.el.addEventListener(this.move, this.touchMove, false);
      this.el.addEventListener(this.end, this.touchEnd, false);
      this.el.addEventListener(this.cancel, this.touchEnd, false);
      this.transitionProperty = 'rotate';
      if (Modernizr.csstransforms3d) {
        return this.transitionProperty = 'rotate3d';
      }
    };

    Swipe2Refresh.prototype.undelegate = function() {
      this.el.removeEventListener(this.start, this.touchStart, false);
      this.el.removeEventListener(this.move, this.touchMove, false);
      this.el.removeEventListener(this.end, this.touchEnd, false);
      return this.el.removeEventListener(this.cancel, this.touchEnd, false);
    };


    /**
    		 * Prevent default events and stopPropagation if needed
    		 * @param {event}
     */

    Swipe2Refresh.prototype.preventEvents = function(event) {
      event.preventDefault();
      return event.stopPropagation();
    };

    Swipe2Refresh.prototype.trigger = function() {
      if (!this.triggered) {
        this.triggered = true;
        return this.onSwipeTop();
      }
    };


    /**
    		 * When the touch begins we want to reset the direction. We also want to track the 
    		 * initial position of the touch as we'll be using that to calculate the delta as
    		 * their finger moves.
    		 * @param {event}
     */

    Swipe2Refresh.prototype.touchStart = function(event) {
      var ref, ref1;
      this.triggered = false;
      this.startTime = event.timeStamp;
      this.startX = (ref = event.clientX) != null ? ref : event.touches[0].pageX;
      this.startY = (ref1 = event.clientY) != null ? ref1 : event.touches[0].pageY;
      this.endX = this.startX;
      this.endY = this.startY;
      this.startYPercent = Math.round(this.startY * 100 / this.screenHeight);
      return this.active = this.startYPercent < this.swipeStartPositionPercent;
    };


    /**
    		 * touchMove
    		 * @param {event}
     */

    Swipe2Refresh.prototype.touchMove = function(event) {
      var currentScreen, menu, wrapper, xdelta, yMovement, ydelta;
      if (!this.active) {
        return;
      }
      wrapper = document.querySelector(this._wrapperSelector);
      if (wrapper) {
        wrapper.scrollTop += rect.getBoundingClientRect().top - this._topOffset;
      }
      currentScreen = document.querySelector('.js-current-screen > div');
      menu = document.querySelector('.js-menu');
      if (currentScreen && currentScreen.scrollTop === 0 && menu && menu.scrollTop === 0 && !$(currentScreen).hasClass('dont-reload')) {
        this.endX = event.clientX || event.touches[0].pageX;
        this.endY = event.clientY || event.touches[0].pageY;
        xdelta = Math.abs(this.moveDistanceX());
        ydelta = Math.abs(this.moveDistanceY());
        yMovement = ydelta > xdelta;
        if (yMovement && !this.triggered) {
          if (this.swipeMaxPercent && this.movePercentY() > this.swipeMaxPercent) {
            this.trigger();
            return this.removeTransition();
          } else if (this.movePercentY() > this.swipeMinPercent) {
            return this.transition();
          }
        }
      }
    };


    /**
    		 * touchEnd
    		 * @param {event}
     */

    Swipe2Refresh.prototype.touchEnd = function(event) {
      if (this.active && this.movePercentY() > this.swipeReloadPercent) {
        this.trigger();
      }
      return this.removeTransition();
    };


    /**
    		 * touchCancel
    		 * @param {event}
     */

    Swipe2Refresh.prototype.touchCancel = function(event) {
      if (this.active && this.movePercentY() > this.swipeReloadPercent) {
        this.trigger();
      }
      return this.removeTransition();
    };


    /**
    		 * Transitions from min to max value
    		 * @param {event}
     */

    Swipe2Refresh.prototype.transition = function() {
      var dy, margin, size;
      if (this.triggered) {
        this.removeTransition();
        return;
      }
      dy = this.transitionOffset > this.moveDistanceY() ? 0 : this.moveDistanceY() - this.transitionOffset;
      size = Math.min(Math.round(this.loader.parentNode.offsetWidth / 2), dy);
      this.loader.parentNode.style.display = 'block';
      this.loader.style.width = (size || 0) + 'px';
      this.loader.style.height = (size || 0) + 'px';
      margin = Math.round((size - this.loader.offsetHeight) / 2);
      margin = Math.min(Math.max(0, margin), 30);
      this.loader.style.marginTop = margin + 'px';
      return this.loader.style.marginBottom = margin + 'px';
    };


    /**
    		 * Remove Transitions
    		 * @param {event}
     */

    Swipe2Refresh.prototype.removeTransition = function() {
      this.loader.parentNode.style.display = 'none';
      this.loader.style.width = '0px';
      this.loader.style.height = '0px';
      this.loader.style.marginTop = '0px';
      return this.loader.style.marginBottom = '0px';
    };


    /**
    		 * Calculate touch X distance
    		 * @param {event}
     */

    Swipe2Refresh.prototype.moveDistanceX = function() {
      return this.endX - this.startX;
    };

    Swipe2Refresh.prototype.movePercentX = function() {
      var d;
      d = this.moveDistanceX();
      return Math.round(d * 100 / this.screenWidth);
    };


    /**
    		 * Calculate touch Y distance
    		 * @param {event}
     */

    Swipe2Refresh.prototype.moveDistanceY = function() {
      return this.endY - this.startY;
    };

    Swipe2Refresh.prototype.movePercentY = function() {
      var res;
      res = Math.round(this.moveDistanceY() * 100 / this.screenHeight);
      return res;
    };

    return Swipe2Refresh;

  })();
});
